import { forceCenter, forceCollide, forceLink, forceManyBody, forceRadial, forceSimulation } from 'd3';
import { GraphType, LinkType, NodeType } from '../Types';
import { useEffect } from 'react';

export const simulation = forceSimulation<NodeType, LinkType>();

/** StartSimulation starts the d3 forcelink simulation. This has to be called after the simulation is initialized. */
export function startSimulation(graph: GraphType, windowSize: { width: number; height: number }): void {
  simulation
    .alpha(1.8)
    .force(
      'link',
      forceLink()
        .id((d: any) => d.id)
        .strength(1)
        .distance(25)
    )
    .force('charge', forceManyBody().strength(-1))
    .force('center', forceCenter(windowSize.width / 2, windowSize.height / 2).strength(0.5))
    .force('collide', forceCollide().strength(1).radius(5).iterations(1)) // Force that avoids circle overlapping
    .force('attract', forceRadial(50, windowSize.width / 2, windowSize.height / 2).strength(0.005));
  simulation.nodes(graph.nodes);
  simulation.force<d3.ForceLink<NodeType, LinkType>>('link')?.links(graph.links);
  simulation.alphaTarget(0).restart();
}

// export const useNLForce = ({ graph, windowSize }: Props) => {
//   useEffect(() => {
//     if (graph && graph.nodes.length > 0 && graph.links.length > 0 && windowSize.width && windowSize.height) {
//       startSimulation(graph);
//     }
//   }, [graph, windowSize.width, windowSize.height]);

//   function refresh(): void {
//     simulation.restart();
//   }

//   return { simulation, refresh, startSimulation };
// };
