import { GraphQueryResult, useAppDispatch, useGraphQueryResult } from '../../data-access/store';
import React, { LegacyRef, useEffect, useRef, useState } from 'react';
import styled from 'styled-components';
import * as PIXI from 'pixi.js';
import { GraphType, LinkType, NodeType } from './Types';
import { useNLPixi } from './components/NLPixi';
import { getRelatedLinks } from './components/utils';

interface Props {
  loading?: boolean;
  // currentColours: any;
}

const Div = styled.div`
  background-color: red;
  font: 'Arial';
`;

/** Return default radius */
function Radius() {
  return 1;
}

/** This is the interface of the NodeLinkComponentState. */
export interface NodeLinkComponentState {
  graph: GraphType;
  width: number;
  height: number;
  stage: PIXI.Container;
  links: PIXI.Graphics;
  simulation: d3.Simulation<NodeType, LinkType>;
  myRef: React.RefObject<HTMLDivElement>;
  renderer: PIXI.IRenderer;
  dragOffset: any;
  panOffset: any;
  scalexy: number;
}

export const NodeLinkVis = React.memo((props: Props) => {
  const jaccardThreshold = undefined;
  const myRef = useRef<HTMLDivElement>(null);
  const [graph, setGraph] = useState<GraphType>();
  const [windowSize, setWindowSize] = useState({ width: 1000, height: 1000 });
  const [highlightNodes, setHighlightNodes] = useState<NodeType[]>([]);
  const [highlightedLinks, setHighlightedLinks] = useState<LinkType[]>([]);

  const pixi = useNLPixi({
    windowSize: windowSize,
    onClick: (node) => {
      if (graph) {
        //If node is already clicked we should remove it from the list.
        if (highlightNodes.includes(node)) {
          const index = highlightNodes.indexOf(node, 0);
          setHighlightNodes(highlightNodes.splice(index, 1));
        } else {
          //Else add it to the list.
          setHighlightNodes(highlightNodes.concat(node));
        }
        //Update the list of edges that need to be highlighted
        setHighlightedLinks(getRelatedLinks(graph, highlightNodes, jaccardThreshold || -1));
      }
    },
    highlightNodes: highlightNodes,
    currentShortestPathEdges: [],
    highlightedLinks: highlightedLinks,
    jaccardThreshold: jaccardThreshold,
    myDiv: myRef.current,
  });

  const graphQueryResult = useGraphQueryResult();

  useEffect(() => {
    console.debug('update nodelink useEffect', graphQueryResult);
    pixi.refresh(graphQueryResult);
  }, [graphQueryResult]);

  useEffect(() => {
    console.log('loaded NodeLinkVis');
    window.addEventListener('resize', handleResize);
    // nodeLinkViewModel.subscribeToQueryResult();
    // nodeLinkViewModel.setThisVisAsExportable();
    // nodeLinkViewModel.subscribeToAnalyticsData();

    return () => {
      console.log('unloaded NodeLinkVis');
      window.removeEventListener('resize', handleResize);
    };
  }, []);

  useEffect(() => {
    console.debug('loaded NodeLinkVis');
    setWindowSize({ width: myRef?.current?.clientWidth || 1000, height: myRef?.current?.clientHeight || 1000 });
  }, [myRef]);

  const loading = props.loading;

  // FUNCIONS FROM MODEL

  /** When the screen gets resized, resize the node link canvas ViewModel. */
  const handleResize = () => {
    setWindowSize({ width: window.innerWidth, height: window.innerHeight - 6 });
  };

  return (
    <>
      {/* <input
        onChange={(v) =>
          dispatch(changePrimary({ main: v.currentTarget.value }))
        }
        type="color"
        name="head"
        value={theme.palette.primary.main}
      /> */}
      {loading && (
        <div className="w-full h-full flex justify-center self-center items-center">
          <span className="loading loading-spinner loading-lg"></span>
        </div>
      )}
      {!loading && (
        <div className="h-full overflow-hidden">
          <div className="h-full overflow-hidden" ref={myRef}></div>
          {/* <VisConfigPanelComponent> */}
          {/* <NodeLinkConfigPanelComponent
               graph={this.state.graph}
               nlViewModel={this.nodeLinkViewModel}
            /> */}
          {/*</VisConfigPanelComponent>*/}
          {/*<VisConfigPanelComponent isLeft>*/}
          {/*  <AttributesConfigPanel nodeLinkViewModel={this.nodeLinkViewModel} />*/}
          {/* </VisConfigPanelComponent> */}
        </div>
      )}
    </>
  );
});
NodeLinkVis.displayName = 'NodeLinkVis';

export default NodeLinkVis;
